vertex{
	varying vec3 vertex;
	varying mat4 mvm;
	
	void main(){
		mvm = gl_ModelViewMatrix;
		vertex = (mvm*vec4(gl_Vertex.xyz, 1.0)).xyz;
		
		gl_TexCoord[0] = gl_MultiTexCoord0;
		gl_Position = gl_ModelViewProjectionMatrix * vec4(gl_Vertex.xyz, 1.0);
	}
	
};

fragment{
	uniform sampler2D texture;
	
	uniform vec4 pld;
	uniform vec4 plp;
	uniform vec4 pldiff;
	uniform vec4 pls;
	uniform float plsl;
	uniform float pla;
	
	uniform vec4 light_directions[30]; //Directions and strength
	uniform vec4 light_positions[30]; //Positions and spotCutOff(only for spotlights)
	uniform vec4 light_diffuse[30]; //Diffuse color
	uniform vec4 light_specular[30]; //Specular color
	uniform float light_spotLight[30]; //1 if the light is a spot light, 0 if not
	
	uniform float lights;
	
	varying vec3 vertex;
	varying mat4 mvm;
	
	void main(){
		vec4 color = texture2D(texture, gl_TexCoord[0].xy);
		vec4 outputColor = vec4(0.0, 0.0, 0.0, 1.0);
		vec4 ambient = vec4(color.rgb*gl_LightModel.ambient.rgb, color.a);
		
		for(int i = 0; i < int((lights-pla)); i++){
			vec3 lightDir = (mvm*vec4(light_positions[i].xyz, 1.0)).xyz - vertex;
			float lightLength = sqrt(pow(lightDir.x, 2.0) + pow(lightDir.y, 2.0) + pow(lightDir.z, 2.0));
			lightDir = normalize(lightDir);
				
			//Spot light
			if(light_spotLight[i] == 1.0){
				float angle = abs(degrees(acos(dot(normalize((vec4(lightDir, 0.0)*mvm).xyz), normalize(light_directions[i].xyz)))));
				
				if(angle < light_positions[i].w){
					float lightOffset = 1.0-(1.0/(light_positions[i].w-angle));
						
					//Diffuse
					vec3 diffuse = light_diffuse[i].rgb*lightOffset;
					outputColor += vec4(max(vec3(0.0), ((diffuse-vec3(lightLength/light_directions[i].w))*color.rgb)), 1.0);
				} else{
					outputColor += vec4(0.0, 0.0, 0.0, 1.0);
				}
					
			}
			
			//Not spot light
			else{
				outputColor += vec4(max(vec3(0.0), ((light_diffuse[i].rgb-vec3(lightLength/light_directions[i].w))*color.rgb)), 1.0);
			}
			
		}
		
		if(pla == 1.0){
			vec3 lightDir = (mvm*vec4(plp.xyz, 1.0)).xyz - vertex;
			float lightLength = sqrt(pow(lightDir.x, 2.0) + pow(lightDir.y, 2.0) + pow(lightDir.z, 2.0));
			lightDir = normalize(lightDir);
				
			//Spot light
			if(plsl == 1.0){
				float angle = abs(degrees(acos(dot(normalize((vec4(lightDir, 0.0)*mvm).xyz), normalize(pld.xyz)))));
				
				if(angle < plp.w){
					float lightOffset = 1.0-(1.0/(plp.w-angle));
						
					//Diffuse
					vec3 diffuse = pldiff.rgb*lightOffset;
					outputColor += vec4(max(vec3(0.0), ((diffuse-vec3(lightLength/pld.w))*color.rgb)), 1.0);
				} else{
					outputColor += vec4(0.0, 0.0, 0.0, 1.0);
				}
					
			}
			
			//Not spot light
			else{
				outputColor += vec4(max(vec3(0.0), ((pldiff.rgb-vec3(lightLength/pld.w))*color.rgb)), 1.0);
			}
			
		}
		
		//Universal
		outputColor.a = color.a;
			
		//Ambient
		if(ambient.r >= outputColor.r && ambient.g >= outputColor.g && ambient.b >= outputColor.b){
			outputColor.rgb = ambient.rgb;
		}
		
		gl_FragColor = outputColor;
	}
	
};