vertex{
	#version 120
	attribute vec3 attribute0;
	
	varying vec3 normal;
	varying vec3 vertex;
	varying vec4 position;
	
	varying mat4 mvm;
	varying mat3 tbn;
	
	void main(){
		normal = normalize(gl_NormalMatrix * gl_Normal);
		vertex = (gl_ModelViewMatrix*vec4(gl_Vertex.xyz, 1.0)).xyz;
		
		mvm = gl_ModelViewMatrix;
		
		vec3 tan = normalize(gl_NormalMatrix * attribute0);
		tbn = mat3(tan, normalize(cross(tan, normal)), normal);
		
		gl_TexCoord[0] = gl_MultiTexCoord0;
		gl_TexCoord[1] = gl_MultiTexCoord1;
		gl_TexCoord[2] = gl_MultiTexCoord2;
		gl_Position = gl_ModelViewProjectionMatrix  * gl_Vertex;
		position = gl_Position;
	}
	
};

fragment{
	#version 120
	uniform sampler2D texture;
	uniform sampler2D normal_map;
	uniform sampler2D light_map;
	
	uniform vec4 pld;
	uniform vec4 plp;
	uniform vec4 pldiff;
	uniform vec4 pls;
	uniform float plsl;
	uniform float pla;
	
	uniform vec4 light_directions[30]; //Directions and strength
	uniform vec4 light_positions[30]; //Positions and spotCutOff(only for spotlights)
	uniform vec4 light_diffuse[30]; //Diffuse color
	uniform vec4 light_specular[30]; //Specular color
	uniform float light_spotLight[30]; //1 if the light is a spot light, 0 if not
	
	uniform float lights;
	
	varying vec3 normal;
	varying vec3 vertex;
	varying vec4 position;
	
	varying mat4 mvm;
	varying mat3 tbn;
	
	void main(){
		if(position.z > 0.0){
			vec4 color = texture2D(texture, gl_TexCoord[0].xy);
			vec3 mapNormal = normalize(((texture2D(normal_map, gl_TexCoord[1].xy) * 2.0) - 1.0).xyz);
			vec4 mapLight = texture2D(light_map, gl_TexCoord[2].xy);
			vec3 ambient = (color.rgb*gl_LightModel.ambient.rgb)*mapLight.r;
			
			vec4 outputColor = vec4(0.0, 0.0, 0.0, 1.0);
			vec3 normalFin = tbn*mapNormal.xyz;
			
			for(int i = 0; i < int((lights-pla)); i++){
				vec3 lightDir = (mvm*vec4(light_positions[i].xyz, 1.0)).xyz - vertex;
				float lightLength = sqrt(pow(lightDir.x, 2.0) + pow(lightDir.y, 2.0) + pow(lightDir.z, 2.0));
				lightDir = normalize(lightDir);
				
				vec3 e = normalize(-vertex);
				vec3 reflected = normalize(-reflect(lightDir, normalFin));
				
				//Spot light
				if(light_spotLight[i] == 1.0){
					float angle = abs(degrees(acos(dot(normalize(lightDir*mat3(gl_ModelViewMatrix)), normalize(light_directions[i].xyz)))));
					
					if(angle < light_positions[i].w){
						float lightOffset = 1.0-(1.0/(light_positions[i].w-angle));
						
						//Diffuse
						vec4 diffuse = vec4(0.0);
						if(mapLight.g != 0.0){
							diffuse = light_diffuse[i] * max(dot(normalFin, lightDir), 0.0);
				   			diffuse = clamp(diffuse, 0.0, 1.0)*mapLight.g*lightOffset;
						}
						
						//Specular
						vec4 specular = vec4(0.0);
						if(mapLight.b != 0.0){
							specular = light_specular[i] * pow(max(dot(reflected, e), 0.0), 0.3 * gl_FrontMaterial.shininess);
				   			specular = clamp(specular, 0.0, 1.0)*mapLight.b;
						}
						
						outputColor += vec4(max(vec3(0.0), ((diffuse.rgb-vec3(lightLength/light_directions[i].w))*color.rgb)) + max(vec3(0.0), (specular.rgb-vec3(lightLength/light_directions[i].w))), 1.0);
					} else{
						outputColor += vec4(0.0, 0.0, 0.0, 1.0);
					}
					
				} 
				
				//Not spot light
				else {
					//Diffuse
					vec4 diffuse = vec4(0.0);
					if(mapLight.g != 0.0){
						diffuse = light_diffuse[i] * max(dot(normalFin, lightDir), 0.0);
			   			diffuse = clamp(diffuse, 0.0, 1.0)*mapLight.g;
					}
					
					//Specular
					vec4 specular = vec4(0.0);
					if(mapLight.b != 0.0){
						specular = light_specular[i] * pow(max(dot(reflected, e), 0.0), 0.3 * gl_FrontMaterial.shininess);
			   			specular = clamp(specular, 0.0, 1.0)*mapLight.b;
					}
					
					outputColor += vec4(max(vec3(0.0), ((diffuse.rgb-vec3(lightLength/light_directions[i].w))*color.rgb)) + max(vec3(0.0), (specular.rgb-vec3(lightLength/light_directions[i].w))), 1.0);
				}
				
			}
			
			//Player light
			if(pla == 1.0){
				vec3 lightDir = (mvm*vec4(plp.xyz, 1.0)).xyz - vertex;
				float lightLength = sqrt(pow(lightDir.x, 2.0) + pow(lightDir.y, 2.0) + pow(lightDir.z, 2.0));
				lightDir = normalize(lightDir);
				
				vec3 e = normalize(-vertex);
				vec3 reflected = normalize(-reflect(lightDir, normalFin));
				
				//Spot light
				if(plsl == 1.0){
					float angle = abs(degrees(acos(dot(normalize(lightDir*mat3(gl_ModelViewMatrix)), normalize(pld.xyz)))));
						
					if(angle < plp.w){
						float lightOffset = 1.0-(1.0/(plp.w-angle));
							
						//Diffuse
						vec4 diffuse = vec4(0.0);
						if(mapLight.g != 0.0){
							diffuse = pldiff * max(dot(normalFin, lightDir), 0.0);
					   		diffuse = clamp(diffuse, 0.0, 1.0)*mapLight.g*lightOffset;
						}
							
						//Specular
						vec4 specular = vec4(0.0);
						if(mapLight.b != 0.0){
							specular = pls * pow(max(dot(reflected, e), 0.0), 0.3 * gl_FrontMaterial.shininess);
					   		specular = clamp(specular, 0.0, 1.0)*mapLight.b;
						}
						
						outputColor += vec4(max(vec3(0.0), ((diffuse.rgb-vec3(lightLength/pld.w))*color.rgb)) + max(vec3(0.0), (specular.rgb-vec3(lightLength/pld.w))), 1.0);
					} else{
						outputColor += vec4(0.0, 0.0, 0.0, 1.0);
					}
					
				} 
					
				//Not spot light
				else{
					//Diffuse
					vec4 diffuse = vec4(0.0);
					if(mapLight.g != 0.0){
						diffuse = pldiff * max(dot(normalFin, lightDir), 0.0);
				   		diffuse = clamp(diffuse, 0.0, 1.0)*mapLight.g;
					}
					
					//Specular
					vec4 specular = vec4(0.0);
					if(mapLight.b != 0.0){
						specular = pls * pow(max(dot(reflected, e), 0.0), 0.3 * gl_FrontMaterial.shininess);
				   		specular = clamp(specular, 0.0, 1.0)*mapLight.b;
					}
					
					outputColor += vec4(max(vec3(0.0), ((diffuse.rgb-vec3(lightLength/pld.w))*color.rgb)) + max(vec3(0.0), (specular.rgb-vec3(lightLength/pld.w))), 1.0);
				}
				
			}
			
			//Universal
			outputColor.a = color.a;
			
			//Ambient
			if(mapLight.r != 0.0){
				if(ambient.r >= outputColor.r && ambient.g >= outputColor.g && ambient.b >= outputColor.b){
					outputColor.rgb = ambient.rgb;
				}
				
			}
			
			if(mapLight.r == 0.0 && mapLight.g == 0.0 && mapLight.b == 0.0){
				outputColor = color;
			}
			
			gl_FragColor = outputColor;
		} else{
			gl_FragColor = vec4(1.0);
		}
		
	}
	
};