vertex{
	//Made in 2015-05-03 by Philip R for S.E.G.
	
	uniform mat4 wtm;
	
	varying vec3 normal;
	varying vec3 vertex;
	varying vec4 position;
	
	void main(){
		normal = normalize(gl_NormalMatrix * gl_Normal);
		vertex = (mat3(gl_ModelViewMatrix)*gl_Vertex.xyz).xyz;
		
		gl_TexCoord[0] = gl_MultiTexCoord0;
		gl_TexCoord[1] = gl_MultiTexCoord1;
		gl_TexCoord[2] = gl_MultiTexCoord2;
		gl_Position = gl_ModelViewProjectionMatrix  * (wtm * gl_Vertex);
		position = gl_Position;
	}
	
};

fragment{
	
	uniform sampler2D texture;
	uniform sampler2D normal_map;
	uniform sampler2D light_map;
	
	uniform vec4 light_directions[30]; //Directions and strength
	uniform vec4 light_positions[30]; //Positions and spotCutOff(only for spotlights)
	uniform vec4 light_diffuse[30]; //Diffuse color
	uniform vec4 light_specular[30]; //Specular color
	uniform float light_spotLight[30]; //1 if the light is a spot light, 0 if not
	
	uniform float lights;
	
	varying vec3 normal;
	varying vec3 vertex;
	varying vec4 position;
	
	void main(){
		if(position.z > 0.0){
			vec4 color = texture2D(texture, gl_TexCoord[0].xy);
			vec3 mapNormal = normalize(((texture2D(normal_map, gl_TexCoord[1].xy) * 2.0) - 1.0).xyz);
			vec4 mapLight = texture2D(light_map, gl_TexCoord[2].xy);
			vec3 ambient = (color.rgb*gl_LightModel.ambient.rgb)*mapLight.r;
			
			vec4 outputColor = vec4(0.0, 0.0, 0.0, 1.0);
			vec3 normalFin = normalize(normal+mapNormal);
			
			for(int i = 0; i < int(lights); i++){
				vec3 lightDir = light_positions[i].xyz - vertex;
				float lightLength = sqrt(pow(lightDir.x, 2.0) + pow(lightDir.y, 2.0) + pow(lightDir.z, 2.0));
				lightDir = normalize(lightDir);
				
				vec3 e = normalize(-vertex);
				vec3 reflected = normalize(-reflect(lightDir, normalFin));
				
				//Spot light
				if(light_spotLight[i] == 1.0){
					float angle = abs(degrees(acos(dot(normalize(lightDir*mat3(gl_ModelViewMatrix)), normalize(light_directions[i].xyz)))));
					
					if(angle < light_positions[i].w){
						float lightOffset = 1.0-(1.0/(light_positions[i].w-angle));
						
						//Diffuse
						vec4 diffuse = vec4(0.0);
						if(mapLight.g != 0.0){
							diffuse = light_diffuse[i] * max(dot(normalFin, lightDir), 0.0);
				   			diffuse = clamp(diffuse, 0.0, 1.0)*mapLight.g*lightOffset;
						}
						
						//Specular
						vec4 specular = vec4(0.0);
						if(mapLight.b != 0.0){
							specular = light_specular[i] * pow(max(dot(reflected, e), 0.0), 0.3 * gl_FrontMaterial.shininess);
				   			specular = clamp(specular, 0.0, 1.0)*mapLight.b;
						}
						
						outputColor += vec4((color.rgb * ((diffuse.rgb-(lightLength/light_directions[i].w))/lights)) + (specular.rgb-(lightLength/light_directions[i].w)), (color.a/lights));
					} else{
						outputColor += vec4(0.0, 0.0, 0.0, 1.0);
					}
					
				} 
				
				//Not spot light
				else{
					//Diffuse
					vec4 diffuse = vec4(0.0);
					if(mapLight.g != 0.0){
						diffuse = light_diffuse[i] * max(dot(normalFin, lightDir), 0.0);
			   			diffuse = clamp(diffuse, 0.0, 1.0)*mapLight.g;
					}
					
					//Specular
					vec4 specular = vec4(0.0);
					if(mapLight.b != 0.0){
						specular = light_specular[i] * pow(max(dot(reflected, e), 0.0), 0.3 * gl_FrontMaterial.shininess);
			   			specular = clamp(specular, 0.0, 1.0)*mapLight.b;
					}
					
					outputColor += vec4((color.rgb * ((diffuse.rgb-(lightLength/light_directions[i].w))/lights)) + (specular.rgb-(lightLength/light_directions[i].w)), (color.a/lights));
				}
				
			}
			outputColor.a = color.a;
			
			//Ambient
			if(mapLight.r != 0.0){
				if(ambient.r >= outputColor.r && ambient.g >= outputColor.g && ambient.b >= outputColor.b){
					outputColor.rgb = ambient.rgb;
				}
				
			}
			
			if(mapLight.r == 0.0 && mapLight.g == 0.0 && mapLight.b == 0.0){
				outputColor = color;
			}
			
			gl_FragColor = outputColor;
		} else{
			gl_FragColor = vec4(1.0);
		}
		
	}
	
};